@@ -31,7 +31,7 @@ from time import gmtime, strftime
 
 from swiftclient import RequestException
 from swiftclient.utils import config_true_value, generate_temp_url, \
-    prt_bytes, JSONableIterable
+    prt_bytes, parse_timeout, JSONableIterable
 from swiftclient.multithreading import OutputManager
 from swiftclient.exceptions import ClientException
 from swiftclient import __version__ as client_version
@@ -979,7 +979,8 @@ st_upload_options = '''[--changed] [--skip-identical] [--segment-size <size>]
                     [--segment-container <container>] [--leave-segments]
                     [--object-threads <thread>] [--segment-threads <threads>]
                     [--meta <name:value>] [--header <header>] [--use-slo]
-                    [--ignore-checksum] [--object-name <object-name>]
+                    [--use-dlo] [--ignore-checksum] [--skip-container-put]
+                    [--object-name <object-name>]
                     <container> <file_or_directory> [<file_or_directory>] [...]
 '''
 
@@ -1023,13 +1024,18 @@ Optional arguments:
                         repeated. Example: -H "content-type:text/plain"
                          -H "Content-Length: 4000".
   --use-slo             When used in conjunction with --segment-size it will
-                        create a Static Large Object instead of the default
-                        Dynamic Large Object.
+                        create a Static Large Object. Deprecated; this is now
+                        the default behavior when the cluster supports it.
+  --use-dlo             When used in conjunction with --segment-size it will
+                        create a Dynamic Large Object. May be useful with old
+                        swift clusters.
+  --ignore-checksum     Turn off checksum validation for uploads.
+  --skip-container-put  Assume all necessary containers already exist; don't
+                        automatically try to create them.
   --object-name <object-name>
                         Upload file and name object to <object-name> or upload
                         dir and use <object-name> as object prefix instead of
                         folder name.
-  --ignore-checksum     Turn off checksum validation for uploads.
 '''.strip('\n')
 
 
@@ -1044,6 +1050,10 @@ def st_upload(parser, args, output_manager, return_parser=False):
         '--skip-identical', action='store_true', dest='skip_identical',
         default=False, help='Skip uploading files that are identical on '
         'both sides.')
+    parser.add_argument(
+        '--skip-container-put', action='store_true', dest='skip_container_put',
+        default=False, help='Assume all necessary containers already exist; '
+        "don't automatically try to create them.")
     parser.add_argument(
         '-S', '--segment-size', dest='segment_size', help='Upload files '
         'in segments no larger than <size> (in Bytes) and then create a '
@@ -1080,10 +1090,13 @@ def st_upload(parser, args, output_manager, return_parser=False):
         ' This option may be repeated. Example: -H "content-type:text/plain" '
         '-H "Content-Length: 4000"')
     parser.add_argument(
-        '--use-slo', action='store_true', default=False,
+        '--use-slo', action='store_true', default=None,
         help='When used in conjunction with --segment-size, it will '
-        'create a Static Large Object instead of the default '
-        'Dynamic Large Object.')
+        'create a Static Large Object.')
+    parser.add_argument(
+        '--use-dlo', action='store_false', dest="use_slo", default=None,
+        help='When used in conjunction with --segment-size, it will '
+        'create a Dynamic Large Object.')
     parser.add_argument(
         '--object-name', dest='object_name',
         help='Upload file and name object to <object-name> or upload dir and '
@@ -1155,7 +1168,7 @@ def st_upload(parser, args, output_manager, return_parser=False):
         return
 
     if from_stdin:
-        if not options['use_slo']:
+        if options['use_slo'] is None:
             options['use_slo'] = True
         if not options['segment_size']:
             options['segment_size'] = DEFAULT_STDIN_SEGMENT
@@ -1381,14 +1394,16 @@ Positional arguments:
   <method>              An HTTP method to allow for this temporary URL.
                         Usually 'GET' or 'PUT'.
   <time>                The amount of time the temporary URL will be
-                        valid. The time can be specified in two ways:
-                        an integer representing the time in seconds or an
-                        ISO 8601 timestamp in a specific format.
-                        If --absolute is passed and time
-                        is an integer, the seconds are intepreted as the Unix
-                        timestamp when the temporary URL will expire. The ISO
-                        8601 timestamp can be specified in one of following
-                        formats:
+                        valid. The time can be specified in three ways:
+                        an integer representing the time in seconds;
+                        a number with a 's', 'm', 'h', or 'd' suffix to specify
+                        the time in seconds, minutes, hours, or days; or
+                        an ISO 8601 timestamp in a specific format.
+                        If --absolute is passed and time is an integer, the
+                        seconds are intepreted as the Unix timestamp when the
+                        temporary URL will expire.
+                        The ISO 8601 timestamp can be specified in one of
+                        following formats:
 
                         i) Complete date: YYYY-MM-DD (eg 1997-07-16)
 
@@ -1737,7 +1752,7 @@ def add_default_args(parser):
     parser.add_argument('-K', '--key', dest='key',
                         default=environ.get('ST_KEY'),
                         help='Key for obtaining an auth token.')
-    parser.add_argument('-T', '--timeout', type=int, dest='timeout',
+    parser.add_argument('-T', '--timeout', type=parse_timeout, dest='timeout',
                         default=None,
                         help='Timeout in seconds to wait for response.')
     parser.add_argument('-R', '--retries', type=int, default=5, dest='retries',
@@ -1938,8 +1953,6 @@ def add_default_args(parser):
 def main(arguments=None):
     argv = sys_argv if arguments is None else arguments
 
-    argv = [a if isinstance(a, str) else a.decode('utf-8') for a in argv]
-
     parser = argparse.ArgumentParser(
         add_help=False, formatter_class=HelpFormatter, usage='''
 %(prog)s [--version] [--help] [--os-help] [--snet] [--verbose]
